@@ -173,25 +173,31 @@ class TaskService(ClientBase):
                 default_worker_id,
             )
         )
-        while True:
-            polled = self.batch_poll(task_type, thread_count, timeout=1, domain=domain)
-            with pool.ThreadPool(thread_count) as tp:
+        with pool.ThreadPool(thread_count) as tp:
+            job_results = []
+            while True:
+                for job in job_results:
+                    if job._number_left == 0:
+                        job_results.remove(job)
+                running_jobs = sum([job._number_left for job in job_results])
+                poll_amount = thread_count - running_jobs
+                print(f"looking for {poll_amount} jobs")
+                polled = self.batch_poll(
+                    task_type, poll_amount, timeout=1, domain=domain
+                )
+                tasks = []
                 while 1:
                     try:
-                        task = polled.next()
+                        tasks.append((polled.next(), self, exec_function))
                     except StopIteration:
                         break
-                    tp.apply_async(
-                        ack_and_execute,
-                        args=(
-                            task,
-                            self,
-                            exec_function,
-                        ),
-                    )
-                tp.close()
-                tp.join()
-            time.sleep(float(polling_interval))
+                    except Exception:
+                        continue
+                print(f" found {len(tasks)} jobs")
+                if tasks:
+                    job_results.append(tp.map_async(ack_and_execute, tasks))
+                else:
+                    time.sleep(float(polling_interval))
 
     def start(
         self,
@@ -228,6 +234,7 @@ class TaskService(ClientBase):
                 time.sleep(1)
 
 
-def ack_and_execute(task, taskClient: TaskService, exec_function):
+def ack_and_execute(args):
+    task, taskClient, exec_function = args
     taskClient.ack_task(task.task_id)
     taskClient.execute(task, exec_function)
